/*
 *  $Id: group_tranks.c,v 1.3 1993/11/12 12:00:48 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

/*@

MPI_Group_translate_ranks - 

@*/

#include <mpi.h>


int MPI_Group_translate_ranks ( group_a, n, ranks_a, group_b, ranks_b )
MPI_Group group_a;
int n;
uint32 *ranks_a;
MPI_Group group_b;
uint32 **ranks_b;
{
  int i, j;
  int pid_a, rank_a;

  /* Check for valid number of ranks, valid ranks_a, and valid ranks_b */
  if ( n<=0 )
    return (MPI_SUCCESS);

  /* Set ranks_b array to MPI_UNDEFINED */
  for ( i=0; i<n; i++ )
    (*ranks_b)[i] = MPI_UNDEFINED;
 
  /* If group is NULL, exit with MPI_SUCCESS, ranks_b is MPI_UNDEFINED */
  if ( (group_a == (MPI_Group)0) || (group_b == (MPI_Group)0) ||
       (ranks_a == (uint32 *)0) ) 
    return (MPI_SUCCESS);

  /* Translate the ranks in ranks_a to ranks_b */
  for ( i=0 ; i<n; i++ ) {
    if ( ((rank_a = ranks_a[i]) >= group_a->np) || (rank_a < 0) ) 
      break;
    pid_a = group_a->lrank_to_grank[rank_a];
    for ( j=0; j<group_b->np; j++ ) 
      if ( pid_a == group_b->lrank_to_grank[j] ) {
        (*ranks_b)[i] = j;
        break;
      }
  }
  return (MPI_SUCCESS);
}
