/*
 *  $Id: datatypes.c,v 1.4 1993/11/22 05:44:26 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include "mpi.h"
#include "mpisys.h"

/*@
    MPI_Type_extent - 
@*/
MPI_Type_extent( datatype, extent )
MPI_Datatype datatype;
int          *extent;
{
    *extent = datatype->extent;
}

/*@
    MPI_Type_Contiguous - 
@*/
MPI_Type_Contiguous( count, oldtype, newtype )
int          count;
MPI_Datatype oldtype;
MPI_Datatype *newtype;
{
    MPIR_DATATYPE  *dteptr;

    *newtype          = (MPI_Datatype) SBalloc( MPIR_dtes );
    dteptr            = *newtype;
    dteptr->dte_type  = MPIR_CONTIG;
    dteptr->committed = NO;
    dteptr->extent    = count * oldtype->extent;
    dteptr->use_count = 0;
    dteptr->old_type  = oldtype;
    dteptr->count     = count;
}

/*@
    MPI_Type_Vector - 
@*/
MPI_Type_Vector( count, oldtype, stride, blocklen, newtype )
int          count;
MPI_Datatype oldtype;
int 	     stride;
int 	     blocklen;
MPI_Datatype *newtype;
{
    MPIR_DATATYPE  *dteptr;

    *newtype          = (MPI_Datatype) SBalloc( MPIR_dtes );
    dteptr            = *newtype;
    dteptr->dte_type  = MPIR_VECTOR;
    dteptr->committed = NO;
    dteptr->use_count = 0;
    dteptr->old_type  = oldtype;
    dteptr->count     = count;
    dteptr->stride    = stride;
    dteptr->blocklen  = blocklen;
}

/*@
    MPI_Type_Hvector - 
@*/
MPI_Type_Hvector( count, oldtype, stride, blocklen, newtype )
int          count;
MPI_Datatype oldtype;
int 	     stride;
int 	     blocklen;
MPI_Datatype *newtype;
{
    MPIR_DATATYPE  *dteptr;

    *newtype          = (MPI_Datatype) SBalloc( MPIR_dtes );
    dteptr            = *newtype;
    dteptr->dte_type  = MPIR_HVECTOR;
    dteptr->committed = NO;
    dteptr->use_count = 0;
    dteptr->old_type  = oldtype;
    dteptr->count     = count;
    dteptr->stride    = stride;
    dteptr->blocklen  = blocklen;
}

/*@
    MPI_Type_Indexed - 
@*/
MPI_Type_Indexed( count, oldtype, indices, blocklens, newtype )
int          count;
MPI_Datatype oldtype;
int 	     indices[];
int 	     blocklens[];
MPI_Datatype *newtype;
{
    MPIR_DATATYPE  *dteptr;
    int            i;

    *newtype          = (MPI_Datatype) SBalloc( MPIR_dtes );
    dteptr            = *newtype;
    dteptr->dte_type  = MPIR_INDEXED;
    dteptr->committed = NO;
    dteptr->use_count = 0;
    dteptr->old_type  = oldtype;
    dteptr->count     = count;
    dteptr->indices   = ( int * ) MPIR_MALLOC( count * sizeof( int ) );
    dteptr->blocklens = ( int * ) MPIR_MALLOC( count * sizeof( int ) );
    for (i = 0; i < count; i++)
    {
	dteptr->indices[i]   = indices[i];
	dteptr->blocklens[i] = blocklens[i];
    }
}

/*@
    MPI_Type_Hindexed - 
@*/
MPI_Type_Hindexed( count, oldtype, indices, blocklens, newtype )
int          count;
MPI_Datatype oldtype;
int 	     indices[];
int 	     blocklens[];
MPI_Datatype *newtype;
{
    MPIR_DATATYPE  *dteptr;
    int            i;

    *newtype          = (MPI_Datatype) SBalloc( MPIR_dtes );
    dteptr            = *newtype;
    dteptr->dte_type  = MPIR_HINDEXED;
    dteptr->committed = NO;
    dteptr->use_count = 0;
    dteptr->old_type  = oldtype;
    dteptr->count     = count;
    dteptr->indices   = ( int * ) MPIR_MALLOC( count * sizeof( int ) );
    dteptr->blocklens = ( int * ) MPIR_MALLOC( count * sizeof( int ) );
    for (i = 0; i < count; i++)
    {
	dteptr->indices[i]   = indices[i];
	dteptr->blocklens[i] = blocklens[i];
    }
}

/*@
    MPI_Type_Struct - 
@*/
MPI_Type_Struct( count, oldtypes, indices, blocklens, newtype )
int          count;
MPI_Datatype oldtypes[];
int 	     indices[];
int 	     blocklens[];
MPI_Datatype *newtype;
{
    MPIR_DATATYPE  *dteptr;
    int            i;

    *newtype          = (MPI_Datatype) SBalloc( MPIR_dtes );
    dteptr            = *newtype;
    dteptr->dte_type  = MPIR_STRUCT;
    dteptr->committed = NO;
    dteptr->use_count = 0;
    dteptr->count     = count;
    dteptr->indices   = ( int * ) MPIR_MALLOC( count * sizeof( int ) );
    dteptr->blocklens = ( int * ) MPIR_MALLOC( count * sizeof( int ) );
    dteptr->old_types = ( MPIR_DATATYPE ** )
	                        MPIR_MALLOC( count * sizeof( MPIR_DATATYPE *));
    for (i = 0; i < count; i++)
    {
	dteptr->indices[i]   = indices[i];
	dteptr->blocklens[i] = blocklens[i];
	dteptr->old_types[i] = oldtypes[i];
    }
}
