/*
 *  $Id: graph_nbr.c,v 1.4 1993/11/12 12:35:46 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

#ifndef MPIR_MIN
#define MPIR_MIN(a,b) ((a)>(b)?(b):(a))
#endif

/*@

MPI_Graph_neighbors - returns the neighbors of a node associated 
                      with a graph topology

@*/
int MPI_Graph_neighbors ( comm, rank, maxneighbors, neighbors )
MPI_Comm  comm;
int               rank;
int               maxneighbors;
int              *neighbors;
{
  int i, begin, end;

  /* Check for valid arguments */
  if ( (comm                      == (MPI_Comm)0)               ||
       (comm->topology.type       != MPI_GRAPH)                         ||
       (rank                      <  0)                                 ||
       (rank                      >= (comm->topology.tag.graph.nnodes)) ||
       (maxneighbors              <= 0)                                 ||
       (neighbors                 == (int *)0) )
    return (MPI_SUCCESS);

  /* Get neighbors */
  if ( rank == 0 ) 
    begin = 0;
  else
    begin = comm->topology.tag.graph.index[rank-1];
  end = comm->topology.tag.graph.index[rank];
  end = MPIR_MIN(end-begin, maxneighbors) + begin;
  for ( i=begin; i<end; i++ )
    neighbors[i-begin] = comm->topology.tag.graph.edges[i];

  return (MPI_SUCCESS);
}
